"""imports"""
from typing import List

from circles_local_database_python.connector import Connector
from circles_local_database_python.generic_crud_ml import GenericCRUDML
from logger_local.Logger import Logger
from logger_local.LoggerComponentEnum import LoggerComponentEnum
from message_local.Recipient import Recipient
from messages_local.MessagesLocal import MessagesLocal

MESSAGE_SEND_PLATFORM_INVITATION_LOCAL_PYTHON_COMPONENT_ID = 243
MESSAGE_SEND_PLATFORM_INVITATION_LOCAL_PYTHON_COMPONENT_COMPONENT_NAME = "message-send-platform-invitation-local-python"
DEVELOPER_EMAIL = 'jenya.b@circ.zone'
object1 = {
    'component_id': MESSAGE_SEND_PLATFORM_INVITATION_LOCAL_PYTHON_COMPONENT_ID,
    'component_name': MESSAGE_SEND_PLATFORM_INVITATION_LOCAL_PYTHON_COMPONENT_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'developer_email': DEVELOPER_EMAIL
}
logger = Logger.create_logger(object=object1)


class CampaignMessageSend(GenericCRUDML):
    """Message send platform class"""

    def __init__(self, schema_name="message", connection: Connector = None) -> None:
        super().__init__(default_schema_name=schema_name, connection=connection)

    def __get_potential_person_list_by_campaign_id_limit(self, campaign_id: int,
                                                         recipient_limit: int = 100) -> List[Recipient]:
        """return list of person id """
        logger.start(f"get potential person list by campaign id={campaign_id}")

        recipient_limit_left = recipient_limit

        query_for_relevant_criteria_for_campaign = """
            SELECT criteria.min_age, criteria.max_age, 
                criteria.gender_list_id, criteria.group_list_id,
                campaign.minimal_days_between_messages_to_the_same_recipient
            FROM campaign.campaign_view AS campaign
               JOIN campaign_criteria.campaign_criteria_view AS campaign_criteria
                   ON campaign_criteria.campaign_id=campaign.campaign_id
               JOIN criteria.criteria_view AS criteria
                   ON criteria.criteria_id = campaign_criteria.criteria_id
            WHERE campaign.campaign_id = %s
        """

        self.cursor.execute(query_for_relevant_criteria_for_campaign, (campaign_id,))
        results = []
        for row in self.cursor.fetchall():
            min_age, max_age, gender_list_id, group_list_id, minimal_days = row
            logger.info(object={"min_age": min_age, "max_age": max_age, "gender_list_id": gender_list_id,
                                "group_list_id": group_list_id, "minimal_days": minimal_days})
            # profile_id didn't receive messages from this campaign for campaign.minimal_days
            where = ("""user.profile_id NOT IN (
                       SELECT user.profile_id FROM message.message_outbox_view 
                           WHERE campaign_id = %s AND updated_timestamp >= NOW() - INTERVAL %s DAY
                       )"""
                     )
            if min_age is not None:
                where += f" AND TIMESTAMPDIFF(YEAR, person_birthday_date, CURDATE()) >= {min_age}"
            if max_age is not None:
                where += f" AND TIMESTAMPDIFF(YEAR, person_birthday_date, CURDATE()) <= {max_age}"

            if gender_list_id is not None:
                where += " AND profile_gender_id " + self.sql_in_list_by_entity_list_id(
                    schema_name="profile", entity_name="gender", entity_list_id=gender_list_id)

            if group_list_id is not None:
                where += " AND group_profile.group_id " + self.sql_in_list_by_entity_list_id(
                    schema_name="`group`", entity_name="group", entity_list_id=group_list_id)

            query_for_potentials_receipients = f"""
                SELECT DISTINCT user_id, person_id, user_main_email, user.profile_id, 
                   profile_phone_full_number_normalized, profile_preferred_lang_code
                 FROM user.user_general_view AS user
                    JOIN group_profile.group_profile_table AS group_profile on group_profile.profile_id = user.profile_id
                  WHERE {where} LIMIT {recipient_limit_left}
                """
            logger.info(object={"query_for_potentials_receipients": query_for_potentials_receipients,
                                "campaign_id": campaign_id, "minimal_days": minimal_days})

            self.cursor.execute(query_for_potentials_receipients, (campaign_id, minimal_days))

            recieved_results = self.cursor.fetchall()
            for (user_id, person_id, user_main_email, profile_id,
                 profile_phone_full_number_normalized, profile_preferred_lang_code) in recieved_results:
                recipient = Recipient(user_id=user_id, person_id=person_id, email_address=user_main_email,
                                      profile_id=profile_id, telephone_number=profile_phone_full_number_normalized,
                                      preferred_language=profile_preferred_lang_code)
                results.append(recipient)
                logger.info(object={"recipient": recipient})

            recipient_limit_left -= len(recieved_results)

        logger.end(f"potential person list by campaign id={campaign_id}",
                   object={"results": results})
        return results

    def __get_number_of_invitations_sent_in_the_last_24_hours(self, campaign_id: int) -> int:
        """return number of invitations"""
        logger.start(
            f"get number of invitations sent in the last 24_hours for campaign id={campaign_id}")
        query = """
            SELECT COUNT(*) FROM message.message_outbox_view 
            WHERE campaign_id = %s 
               AND return_code = 0   -- success
               AND updated_timestamp >= NOW() - INTERVAL 24 HOUR  -- updated in the last 24 hours
            """

        self.cursor.execute(query, (campaign_id,))
        number_of_invitations_tuple = self.cursor.fetchall()
        number_of_invitation = number_of_invitations_tuple[0][0]  # get the only element of the only tuple
        logger.end(f"number_of_invitations={number_of_invitation}")
        return number_of_invitation

    def __get_number_of_invitations_to_send_by_campain_id_multiplier(
            self, campaign_id: int,
            additional_invitations_multiplier: float = 1.01,
            additional_invitations_amount: int = 1) -> int:
        """get number to send after multiplier"""
        logger.start()
        invitations_sent_in_the_last_24_hours = int(self.__get_number_of_invitations_sent_in_the_last_24_hours(
            campaign_id) * additional_invitations_multiplier + additional_invitations_amount)
        logger.end(f"number_of_invitations_to_send={invitations_sent_in_the_last_24_hours}")
        return invitations_sent_in_the_last_24_hours

    def __send_message_by_campaign_id(self, campaign_id: int,
                                      additional_invitations_multiplier: float = 1.01,
                                      additional_invitations_amount: int = 1) -> None:
        logger.start(object={"campaign_id": campaign_id,
                             "additional_invitations_multiplier": additional_invitations_multiplier,
                             "additional_invitations_amount": additional_invitations_amount})
        count = self.__get_number_of_invitations_to_send_by_campain_id_multiplier(
            campaign_id=campaign_id,
            additional_invitations_multiplier=additional_invitations_multiplier,
            additional_invitations_amount=additional_invitations_amount)
        person_list = self.__get_potential_person_list_by_campaign_id_limit(campaign_id, count)
        query = """
            SELECT campaign_table.message_template_id, message_template.message_template_ml_table.sms_body_template
            FROM campaign.campaign_table JOIN message_template.message_template_ml_table
                ON campaign_table.message_template_id = message_template_ml_table.message_template_id
            WHERE campaign_table.campaign_id = %s
        """
        self.cursor.execute(query, (campaign_id,))
        text_template = self.cursor.fetchall()
        #  we have to call the constructor every time, as the work related to body/recipients is done there,
        #  and we have new body & recipients per campaign
        return_value = MessagesLocal().send_scheduled(to_recipients=person_list, original_body=text_template,
                                                      campaign_id=campaign_id)
        logger.end(object={"return_value": return_value})

    def send_to_all_campaigns(self, additional_invitations_multiplier: float = 1.01,
                              additional_invitations_amount: int = 1) -> None:
        """send to all campaigns"""
        logger.start(object={"additional_invitations_multiplier": additional_invitations_multiplier,
                             "additional_invitations_amount": additional_invitations_amount})
        self.cursor.execute("SELECT campaign_id FROM campaign.campaign_table WHERE NOW() >= start_timestamp "
                            "AND (end_timestamp IS NULL OR NOW() <= end_timestamp)")
        campaign_ids_list_of_tuples = self.cursor.fetchall()
        logger.info(object={"campaign_ids_list_of_tuples": campaign_ids_list_of_tuples})
        for campaign_id_tuple in campaign_ids_list_of_tuples:
            self.__send_message_by_campaign_id(campaign_id=campaign_id_tuple[0],
                                               additional_invitations_multiplier=additional_invitations_multiplier,
                                               additional_invitations_amount=additional_invitations_amount)
        logger.end()
